#include "dnsbackend.hh"
#include "dns.hh"
#include "ahuexception.hh"
#include "logger.hh"

class DNSPacket;


/* FIRST PART */
class DemoBackend : public DNSBackend
{
public:
  bool list(int id) {
    return false; // we don't support AXFR
  }
    
  void lookup(const QType &type, const string &qdomain, DNSPacket *p, int zoneId)
  {

    if(type.getCode()!=QType::A || qdomain!="random.powerdns.com")  // we only know about random.powerdns.com A
      d_answer="";                                                  // no answer
    else {
      ostringstream os;
      os<<random()%256<<"."<<random()%256<<"."<<random()%256<<"."<<random()%256;
      d_answer=os.str();                                           // our random ip address
    }
  }

  bool get(DNSResourceRecord &rr)
  {
    if(!d_answer.empty()) {
      rr.qname="random.powerdns.com";                               // fill in details
      rr.qtype=QType::A;                                            // A record
      rr.ttl=86400;                                                 // 1 day
      rr.content=d_answer;

      d_answer="";                                                  // this was the last answer
      
      return true;
    }
    return false;                                                   // no more data
  }
  
private:
  string d_answer;
};

/* SECOND PART */

class DemoFactory : public BackendFactory
{
public:
  DemoFactory() : BackendFactory("demo") {}

  DNSBackend *make(const string &suffix="")
  {
    return new DemoBackend();
  }
};

/* THIRD PART */

class DemoLoader
{
public:
  DemoLoader()
  {
    BackendMakers().report(new DemoFactory);
    
    L<<Logger::Info<<" [DemoBackend] This is the demobackend ("__DATE__", "__TIME__") reporting"<<endl;
  }  
};

static DemoLoader loader;

