#ifndef MISC_HH
#define MISC_HH
/* (C) 2002 POWERDNS.COM BV */

#include <string>
#include <sys/time.h>
#include <time.h>

using namespace std;
int matchNetmask(const char *address, const char *omask);
string humanDuration(time_t passed);
void chomp(string &line, const string &delim);
void stripLine(string &line);
string getHostname();
string urlEncode(const string &text);
int waitForData(int fd, int seconds);

void upperCase(string& s);

struct ServiceTuple
{
  string host;
  u_int16_t port;
};
void parseService(const string &descr, ServiceTuple &st);

template <typename Container>
void
stringtok (Container &container, string const &in,
           const char * const delimiters = " \t\n")
{
  const string::size_type len = in.length();
  string::size_type i = 0;
  
  while (i<len) {
    // eat leading whitespace
    i = in.find_first_not_of (delimiters, i);
    if (i == string::npos)
      return;   // nothing left but white space
    
    // find the end of the token
    string::size_type j = in.find_first_of (delimiters, i);
    
    // push token
    if (j == string::npos) {
      container.push_back (in.substr(i));
      return;
    } else
      container.push_back (in.substr(i, j-i));
    
    // set up for next loop
    i = j + 1;
  }
}


string stringerror();
string itoa(int i);

void dropPrivs(int uid, int gid);
int makeGidNumeric(const string &group);
int makeUidNumeric(const string &user);
void cleanSlashes(string &str);

/** The DTime class can be used for timing statistics with microsecond resolution. 
On 32 bits systems this means that 2147 seconds is the longest time that can be measured. */
class DTime 
{
public:
  DTime(); //!< This constructor calls set() for you so the timer is set on construction
  DTime(const DTime &dt);
  time_t time();
  void set();  //!< Reset the timer
  int udiff(); //!< Return the number of microseconds since the timer was last set.
private:
  struct timeval d_set;
};

#endif
